All of lore.kernel.org
 help / color / mirror / Atom feed
From: "Matthew Wilcox (Oracle)" <willy@infradead.org>
To: linux-mm@kvack.org, linux-fsdevel@vger.kernel.org,
	akpm@linux-foundation.org
Cc: "Matthew Wilcox (Oracle)" <willy@infradead.org>,
	Christoph Hellwig <hch@lst.de>, Jeff Layton <jlayton@kernel.org>
Subject: [PATCH v8 10/27] mm: Handle per-folio private data
Date: Fri, 30 Apr 2021 18:22:18 +0100	[thread overview]
Message-ID: <20210430172235.2695303-11-willy@infradead.org> (raw)
In-Reply-To: <20210430172235.2695303-1-willy@infradead.org>

Add folio_get_private() and folio_set_private() which mirror page_private()
and set_page_private() -- ie folio private data is the same as page
private data.  The only difference is that these return a void *
instead of an unsigned long, which matches the majority of users.

Turn attach_page_private() into folio_attach_private() and reimplement
attach_page_private() as a wrapper.  No filesystem which uses page private
data currently supports compound pages, so we're free to define the rules.
attach_page_private() may only be called on a head page; if you want
to add private data to a tail page, you can call set_page_private()
directly (and shouldn't increment the page refcount!  That should be
done when adding private data to the head page / folio).

This saves 597 bytes of text with the distro-derived config that I'm
testing due to removing the calls to compound_head() in get_page()
& put_page().

Signed-off-by: Matthew Wilcox (Oracle) <willy@infradead.org>
Reviewed-by: Christoph Hellwig <hch@lst.de>
Acked-by: Jeff Layton <jlayton@kernel.org>
---
 include/linux/mm_types.h | 11 +++++++++
 include/linux/pagemap.h  | 48 ++++++++++++++++++++++++----------------
 2 files changed, 40 insertions(+), 19 deletions(-)

diff --git a/include/linux/mm_types.h b/include/linux/mm_types.h
index 276e358c75d3..111c304b7d13 100644
--- a/include/linux/mm_types.h
+++ b/include/linux/mm_types.h
@@ -302,6 +302,12 @@ static inline atomic_t *compound_pincount_ptr(struct page *page)
 #define PAGE_FRAG_CACHE_MAX_SIZE	__ALIGN_MASK(32768, ~PAGE_MASK)
 #define PAGE_FRAG_CACHE_MAX_ORDER	get_order(PAGE_FRAG_CACHE_MAX_SIZE)
 
+/*
+ * page_private can be used on tail pages.  However, PagePrivate is only
+ * checked by the VM on the head page.  So page_private on the tail pages
+ * should be used for data that's ancillary to the head page (eg attaching
+ * buffer heads to tail pages after attaching buffer heads to the head page)
+ */
 #define page_private(page)		((page)->private)
 
 static inline void set_page_private(struct page *page, unsigned long private)
@@ -309,6 +315,11 @@ static inline void set_page_private(struct page *page, unsigned long private)
 	page->private = private;
 }
 
+static inline void *folio_get_private(struct folio *folio)
+{
+	return (void *)folio->private;
+}
+
 struct page_frag_cache {
 	void * va;
 #if (PAGE_SIZE < PAGE_FRAG_CACHE_MAX_SIZE)
diff --git a/include/linux/pagemap.h b/include/linux/pagemap.h
index a4bd41128bf3..c58587225fe5 100644
--- a/include/linux/pagemap.h
+++ b/include/linux/pagemap.h
@@ -260,42 +260,52 @@ static inline int page_cache_add_speculative(struct page *page, int count)
 }
 
 /**
  * attach_page_private - Attach private data to a page.
  * @page: Page to attach data to.
  * @data: Data to attach to page.
  *
  * Attaching private data to a page increments the page's reference count.
  * The data must be detached before the page will be freed.
  */
-static inline void attach_page_private(struct page *page, void *data)
+static inline void folio_attach_private(struct folio *folio, void *data)
 {
-	get_page(page);
-	set_page_private(page, (unsigned long)data);
-	SetPagePrivate(page);
+	folio_get(folio);
+	folio->private = (unsigned long)data;
+	folio_set_private(folio);
 }
 
 /**
  * detach_page_private - Detach private data from a page.
  * @page: Page to detach data from.
  *
  * Removes the data that was previously attached to the page and decrements
  * the refcount on the page.
  *
  * Return: Data that was attached to the page.
  */
-static inline void *detach_page_private(struct page *page)
+static inline void *folio_detach_private(struct folio *folio)
 {
-	void *data = (void *)page_private(page);
+	void *data = folio_get_private(folio);
 
-	if (!PagePrivate(page))
+	if (!folio_private(folio))
 		return NULL;
-	ClearPagePrivate(page);
-	set_page_private(page, 0);
-	put_page(page);
+	folio_clear_private(folio);
+	folio->private = 0;
+	folio_put(folio);
 
 	return data;
 }
 
+static inline void attach_page_private(struct page *page, void *data)
+{
+	folio_attach_private(page_folio(page), data);
+}
+
+static inline void *detach_page_private(struct page *page)
+{
+	return folio_detach_private(page_folio(page));
+}
+
 #ifdef CONFIG_NUMA
 extern struct page *__page_cache_alloc(gfp_t gfp);
 #else
@@ -260,42 +260,52 @@ static inline int page_cache_add_speculative(struct page *page, int count)
 
 /**
- * attach_page_private - Attach private data to a page.
- * @page: Page to attach data to.
- * @data: Data to attach to page.
+ * folio_attach_private - Attach private data to a folio.
+ * @folio: Folio to attach data to.
+ * @data: Data to attach to folio.
  *
@@ -260,42 +260,52 @@ static inline int page_cache_add_speculative(struct page *page, int count)
  *
- * Attaching private data to a page increments the page's reference count.
- * The data must be detached before the page will be freed.
+ * Attaching private data to a folio increments the page's reference count.
+ * The data must be detached before the folio will be freed.
  */
@@ -260,42 +260,52 @@ static inline int page_cache_add_speculative(struct page *page, int count)
 
 /**
- * detach_page_private - Detach private data from a page.
- * @page: Page to detach data from.
+ * folio_detach_private - Detach private data from a folio.
+ * @folio: Folio to detach data from.
  *
@@ -260,42 +260,52 @@ static inline int page_cache_add_speculative(struct page *page, int count)
  *
- * Removes the data that was previously attached to the page and decrements
+ * Removes the data that was previously attached to the folio and decrements
  * the refcount on the page.
  *
@@ -260,42 +260,52 @@ static inline int page_cache_add_speculative(struct page *page, int count)
  * the refcount on the page.
  *
- * Return: Data that was attached to the page.
+ * Return: Data that was attached to the folio.
  */
-- 
2.30.2


  parent reply	other threads:[~2021-04-30 17:31 UTC|newest]

Thread overview: 21+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2021-04-30 17:22 [PATCH v8 00/27] Memory Folios Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 01/27] mm: Introduce struct folio Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 02/27] mm: Add folio_pgdat and folio_zone Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 03/27] mm/vmstat: Add functions to account folio statistics Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 04/27] mm/debug: Add VM_BUG_ON_FOLIO and VM_WARN_ON_ONCE_FOLIO Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 05/27] mm: Add folio reference count functions Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 06/27] mm: Add folio_put Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 07/27] mm: Add folio_get Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 08/27] mm: Add folio flag manipulation functions Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 09/27] mm: Add folio_young() and folio_idle() Matthew Wilcox (Oracle)
2021-04-30 17:22 ` Matthew Wilcox (Oracle) [this message]
2021-04-30 17:22 ` [PATCH v8 11/27] mm/filemap: Add folio_index, folio_file_page and folio_contains Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 12/27] mm/filemap: Add folio_next_index Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 13/27] mm/filemap: Add folio_offset and folio_file_offset Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 14/27] mm/util: Add folio_mapping and folio_file_mapping Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 15/27] mm: Add folio_mapcount Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 16/27] mm/memcg: Add folio wrappers for various functions Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 17/27] mm/filemap: Add folio_unlock Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 18/27] mm/filemap: Add folio_lock Matthew Wilcox (Oracle)
2021-04-30 17:22 ` [PATCH v8 19/27] mm/filemap: Add folio_lock_killable Matthew Wilcox (Oracle)
2021-04-30 17:44 ` [PATCH v8 00/27] Memory Folios Matthew Wilcox

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20210430172235.2695303-11-willy@infradead.org \
    --to=willy@infradead.org \
    --cc=akpm@linux-foundation.org \
    --cc=hch@lst.de \
    --cc=jlayton@kernel.org \
    --cc=linux-fsdevel@vger.kernel.org \
    --cc=linux-mm@kvack.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.